"""
Created on May 4 2020

@author: Melchior du Lac, Joan Hérisson

"""

import csv
from tarfile import open as tf_open
from argparse import ArgumentParser
from requests import get as r_get
from tempfile import NamedTemporaryFile
from shutil import copyfile
from os import path as os_path
from os import makedirs
from os.path import dirname
from .version import __version__

def build_args_parser():
    parser = ArgumentParser('Python wrapper to fetch RetroRules')
    parser = _add_arguments(parser)
    return parser


class Parser:

    def __init__(self):
        self._retrorules_url = \
            'https://retrorules.org/dl/preparsed/rr02/rp2/hs'
        self._rules_path = ""

    def parse_rules(self,
                    rules_file='',
                    rule_type='',
                    diameters='2,4,6,8,10,12,14,16',
                    outdir='./out',
                    outfile=None,
                    output_format='csv'):

        # Check args is here since the method is directly callable.

        # If rules_file is set, it takes precedence on rule_type
        if not rules_file:
            if rule_type:
                if rule_type not in ['all', 'retro', 'forward']:
                    raise ValueError('Cannot detect \'rule_type\' input: '+str(rule_type))
                if self._rules_path == "":
                    self._rules_path = NamedTemporaryFile().name+'/rules'
                rules_file = os_path.join(self._rules_path,
                                          'retrorules_rr02_rp2_hs',
                                          'retrorules_rr02_rp2_flat_'+rule_type+'.csv')
                if not os_path.exists(rules_file):
                    _download(self._retrorules_url, self._rules_path)
            else:
                raise ValueError(
                        "at least one of --rules_file or --rule_type required")

        diameters_list = diameters.split(',')

        outfile_temp = NamedTemporaryFile().name
        try:
            _parse_and_write(rules_file, diameters_list, outfile_temp)
        except ValueError as e:
            raise ValueError(str(e))

        output_format = output_format.lower()
        if not outfile or outfile == '':
            outfile = os_path.basename(os_path.splitext(rules_file)[0]) + \
                      '_d' + '-'.join(diameters_list)
            outfile = _pkg_out(outfile_temp, outdir, outfile, output_format)
            return outdir+'/'+outfile
        else:
            if output_format == 'tar.gz':
                outfile_name = os_path.basename(os_path.splitext(rules_file)[0]) + \
                          '_d' + '-'.join(diameters_list)
                with tf_open(outfile, mode='w:gz') as tf:
                    tf.add(outfile_temp, outfile_name+'.csv')
            else:
                copyfile(outfile_temp, outfile)
            return outfile




def _pkg_out(file_res_name, outdir, outfile_name, output_format):
    makedirs(outdir, exist_ok=True)
    if output_format.lower() == 'tar.gz':
        with tf_open(outdir+'/'+outfile_name+'.tar.gz', mode='w:gz') as tf:
            tf.add(file_res_name, outfile_name+'.csv')
        outfile_name += '.tar.gz'
    else:
        outfile_name += '.csv'
        copyfile(file_res_name, outdir+'/'+outfile_name)
    return outfile_name


def _parse_and_write(infile, diameters, outfile):
    with open(infile, 'r') as rf:
        with open(outfile, 'w') as o:
            rf_csv = csv.reader(rf)
            o_csv = csv.writer(o, delimiter=',', quotechar='"')
            o_csv.writerow(next(rf_csv))
            for row in rf_csv:
                try:
                    if row[4] in diameters:
                        o_csv.writerow(row)
                except ValueError:
                    raise ValueError(
                        'Cannot convert diameter to integer: '+str(row[4]))


def _download(url, path):
    makedirs(path, exist_ok=True)
    r = r_get(url)
    with NamedTemporaryFile() as tempf:
        tempf.write(r.content)
        tar = tf_open(tempf.name, mode="r:gz")
        tar.extractall(path)
        tar.close()


def _add_arguments(parser):
    parser.add_argument('-rf', '--rules-file',
                        type=str,
                        help="rules file to parse")
    parser.add_argument('-rt', '--rule-type',
                        type=str,
                        choices=['all', 'retro', 'forward'],
                        help="rules file to parse")
    parser.add_argument('--outdir',
                        type=str,
                        default='./out',
                        help="folder where result file is written")
    parser.add_argument('--outfile',
                        type=str,
                        help="file where results are written")
    parser.add_argument('-d', '--diameters',
                        type=str,
                        default='2,4,6,8,10,12,14,16',
                        help='diameter of the sphere including the atoms around the reacting center (default is including all values: 2,4,6,8,10,12,14,16). The higher is the diameter, the more specific are the rules')
    parser.add_argument('-of', '--output-format',
                        type=str,
                        choices=['csv', 'tar.gz'],
                        default='csv',
                        help='output file format (default: csv)')
    parser.add_argument('--version', action='version',
                        version='%(prog)s {version}'.format(version=__version__))
    return parser



#!/usr/bin/env python

from os import path, mkdir
from logging import error as logging_error

from rrparser import Parser, build_args_parser


def _cli():
    parser = build_args_parser()
    params = parser.parse_args()

    try:
        return Parser().parse_rules(rules_file=params.rules_file,
                                    rule_type=params.rule_type,
                                    diameters=params.diameters,
                                    outdir=params.outdir,
                                    outfile=params.outfile,
                                    output_format=params.output_format)
    except ValueError as e:
        logging_error(str(e))


if __name__ == '__main__':
    _cli()



from setuptools import setup
from shutil import copyfile

_package = 'rrparser'

_readme = 'README.md'

with open(_package+'/doc/'+_readme, 'r') as f:
    long_description = f.read()
    # line = f.readline()
    # long_description = line+f.read()
    # _package = line.splitlines()[0].split()[1].lower()

required = []
with open(_package+'/doc/requirements.txt', 'r') as f:
    for l in f:
        required += [l]

_release = 'RELEASE'
# extra_files={
#     'release': (_package, [_package+'/doc/'+_release])
# }

# with open(extra_files['release'][1][0], 'r') as f:
with open(_package+'/doc/'+_release, 'r') as f:
    _version = f.readline().split()[0]

setup(
    name=_package,
    version=_version,
    author='Thomas Duigou, Melchior du Lac, Joan Hérisson',
    author_email='joan.herisson@univ-evry.fr',
    description='Reaction Rules Parser',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/brsynth/RRulesParser',
    packages=[_package],
    # package_dir={_package: _package},
    install_requires=required,
    test_suite='discover_tests',
    package_data={_package: ['doc/*']},
#    include_package_data=True,
#    data_files=[v for v in extra_files.values()],
    license='MIT',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.5',
)
